package lz.android.dexlib2.helper;

import org.jf.dexlib2.dexbacked.DexBackedDexFile;
import org.jf.dexlib2.dexbacked.DexBackedMethod;
import org.jf.dexlib2.dexbacked.DexBackedMethodImplementation;
import org.jf.dexlib2.dexbacked.raw.CodeItem;
import org.jf.dexlib2.iface.Method;
import org.jf.dexlib2.iface.MethodImplementation;
import org.jf.dexlib2.iface.instruction.Instruction;
import org.jf.dexlib2.iface.reference.MethodReference;
import org.jf.dexlib2.immutable.ImmutableMethod;
import org.jf.dexlib2.immutable.ImmutableMethodImplementation;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

/**
* Enhancement to the Method Class of the operation.
*/
public class MethodHelper {
    public static DexBackedDexFile mDexfile;

    public static final String[] paramNames = { "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n",
            "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z" };

    /**
* In the starting position of the inserted instruction.
* @param method the method object.
* @param insts to insert the instruction.
* @param regIncrement register increments.Because the inserted command may need to increase the register.
* @return returns a new method object.
*/
    public static Method insertInstructionInStart(Method method, List<Instruction> insts, int regIncrement) {
        MethodImplementation oldmi = method.getImplementation();
        List<Instruction> newInsts = new ArrayList<>();

        // In the starting position of the inserted instruction.
        for (Instruction inst : insts) {
            newInsts.add(inst);
        }

        // Then the original instruction insert.
        for (Instruction inst : oldmi.getInstructions()) {
            newInsts.add(inst);
        }

        int regCount = oldmi.getRegisterCount() + regIncrement;
        ImmutableMethodImplementation newmi = new ImmutableMethodImplementation(regCount, newInsts, oldmi.getTryBlocks(), oldmi.getDebugItems());
        return new ImmutableMethod(method.getDefiningClass(), method.getName(), method.getParameters(), method.getReturnType(), method.getAccessFlags(), method.getAnnotations(), newmi);
    }

    /**
* GET method of instruction.
* @param dexBackedMethod
* @return returns the method in the instruction.
*/
    public static short[] getInstructions(DexBackedMethod dexBackedMethod) {
        int codeOffset = getCodeOffset(dexBackedMethod);//codeItem structure offset
        DexBackedDexFile dexFile = dexBackedMethod.getImplementation().dexFile;
        mDexfile =dexFile;

        // This size refers to the u2 array element number.

        //Tim: instruct the size of the
        int instructionsSize = dexFile.readSmallUint(codeOffset + CodeItem.INSTRUCTION_COUNT_OFFSET);

        int instructionsStartOffset = codeOffset + CodeItem.INSTRUCTION_START_OFFSET;
        short[] insts = new short[instructionsSize];
        for (int i = 0; i < instructionsSize; i++) {
            insts[i] = (short) dexFile.readUshort(instructionsStartOffset);
            instructionsStartOffset += 2;
        }

        return insts;
    }

    /**
* GET method of the code_item structure of the offset.
* @param dexBackedMethod
* @return returns the method of the code_item structure of the offset.
*/
    public static int getCodeOffset(DexBackedMethod dexBackedMethod) {
        int offset = -1;
        DexBackedMethodImplementation dbmi = dexBackedMethod.getImplementation();
        try {
            Field field = dbmi.getClass().getDeclaredField("codeOffset");
            field.setAccessible(true);
            offset = (Integer) field.get(dbmi);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return offset;
    }

    /**
* Generate the method signature.
*
* @param mr
* @return
*/
    public static String genMethodSig(MethodReference mr) {
        List<? extends CharSequence> params = mr.getParameterTypes();
        StringBuilder sb = new StringBuilder();
        sb.append("(");
        int length = params.size();
        for (int i = 0; i < length; i++) {
            sb.append(params.get(i));
        }
        sb.append(")");
        sb.append(mr.getReturnType());
        return sb.toString();
    }

    /**
* Generate method parameter of the short type.
* @param mr
* @return returns the method parameter of the short type.
*/
    public static String genParamsShortDesc(MethodReference mr) {
        List<? extends CharSequence> params = mr.getParameterTypes();
        StringBuilder sb = new StringBuilder();
        int length = params.size();
        /*Print the output parameters of full type*/
        for(int i=0;i<length;i++)
        {
            System.out.println("Parameters"+i+" "+params);
        }
        for (int i = 0; i < length; i++) {
            char chType = params.get(i).charAt(0);
            if ('[' == chType) {
                sb.append('L');
            } else {
                sb.append(chType);
            }
        }
        String test=sb.toString();
        return sb.toString();
    }

    /**
* Generate in the native in type.
* @param mr
* @return
*/
    public static String genTypeInNative (MethodReference mr) {
        String type = mr.getReturnType();
        return genTypeInNative(type);
    }

    /**
* Generate in the native in type.
* @param type
* @return
*/
    public static String genTypeInNative (CharSequence type) {
        char cType = type.charAt(0);
        switch (cType) {
            case 'V':
                return "void";
            case 'Z':
                return "jboolean";
            case 'B':
                return "jbyte";
            case 'S':
                return "jshort";
            case 'C':
                return "jchar";
            case 'I':
                return "jint";
            case 'J':
                return "jlong";
            case 'F':
                return "jfloat";
            case 'D':
                return "jdouble";
            case 'L':
                return "jobject";
            case '[':
                return "jobject";
            default:
                return null;
        }
    }

    /**
* Generate the JNI method parameters list.
*
* @param mr
* @return returns the method parameter list.
*/
    public static String genParamTypeListInNative(MethodReference mr) {
        List<? extends CharSequence> params = mr.getParameterTypes();
        StringBuilder paramsList = new StringBuilder();
        paramsList.append("JNIEnv *env, jobject thiz");
        int length = params.size();

        for (int i = 0; i < length; i++) {
            paramsList.append(", ");
            paramsList.append(genTypeInNative(params.get(i)));
            paramsList.append(" ");
            paramsList.append(paramNames[i]);
        }
        return paramsList.toString();
    }

    /**
* Generate JNINativeMethod structure of the data.
* @param method
* @return
*/
    public static String genJNINativeMethod(Method method) {
        return String.format("{ \"%s\", \"%s\", (void*)%s }, ", method.getName(), genMethodSig(method), method.getName());
    }

}

