package lz.android.shell.base.helper;

import lz.android.dexlib2.helper.MethodHelper;
import lz.android.shell.base.TypeDescription;
import org.jf.dexlib2.iface.ClassDef;
import org.jf.dexlib2.iface.Method;

import javax.swing.plaf.synth.SynthTextAreaUI;
import java.util.Iterator;
import java.util.List;

/**
* Created by Neptunian on 2015/4/1.
*/
public class TypeDescriptionHelper {

/**
* The class the full path of the converted to the TypeDescription type.
* @param fullClassName the class of the full path.
* @return returns the TypeDescription type.
*/
    public static TypeDescription convertByFullClassName(String fullClassName) {
        TypeDescription td = new TypeDescription();
        int lastIndex = fullClassName.lastIndexOf('.');
        if(lastIndex == -1){
            System.out.println("To be protected method of a class name input error, please verify the input, thank you!");
            System.exit(0);
        }
        td.packageName = fullClassName.substring(0, lastIndex);
        td.className = fullClassName.substring(lastIndex + 1);
        return td;
    }

    /**
* Extract the ClassDef of the information, to generate a TypeDescription object.
* @param classDef
* @return
*/
    public static TypeDescription convertByClassDef (ClassDef classDef) {
        String type = classDef.getType();
        type = type.substring(1, type.length() - 1).replace('/', '.');
        return convertByFullClassName(type);
    }

    /**
* Extraction Method information, generating a TypeDescription object.
* @param method
* @return
*/
    public static TypeDescription convertByMethod (Method method) {
        String type = method.getDefiningClass();
        type = type.substring(1, type.length() - 1).replace('/', '.');
        TypeDescription td = convertByFullClassName(type);
        td.methodName = method.getName();
        td.methodSig = MethodHelper.genMethodSig(method);
        return td;
    }

    /**
*
* @param blackList
* @param typeDescription
* @return
*/
public static boolean isMatchBlackList (List<TypeDescription> blackList, TypeDescription typeDescription) {
if (isMatchClassInBlackList(blackList, typeDescription) && isMatchMethodInBlackList(blackList, typeDescription)) {
return true;
} else {
return false;
}
}

public static boolean isMatchWhiteList (List<TypeDescription> whiteList, TypeDescription typeDescription) {
if (isMatchClassInWhiteList(whiteList, typeDescription) && isMatchMethodInWhiteList(whiteList, typeDescription)) {
return true;
} else {
return false;
}
}

/**
* Determine whether a class is included in the White list.If it is the entire package, then it is determined that this class is contained.If the class has an internal class, if this class contains, then this inner class is also included.
*
* @param list
* The list list.
* @param typeDescription
* @return true: include.false: does not contain.
*/
    public static boolean isMatchClassInWhiteList(List<TypeDescription> list, TypeDescription typeDescription) {
        Iterator<TypeDescription> it = list.iterator();
        while (it.hasNext()) {
            TypeDescription td = it.next();

            String packageName = typeDescription.packageName, className = typeDescription.className;

            if (isMatchPackage(td.packageName, packageName)) {
                // If the class name is null, then this item is scoped to the package.
                if (null == td.className) {
                    return true;
                } else {
                    int index_0 = -1;
                    if ("*".equals(td.className)) {
                        return true;
                    } else if ('*' == td.className.charAt(td.className.length() - 1)) {
                        if (0 == className.indexOf(td.className.substring(0, td.className.length() - 1))) {
                            return true;
                        }
                    } else if (td.className.equals(className)) {
                        return true;
                    } else if (-1 != (index_0 = className.indexOf('$'))) {
                        if (className.substring(0, index_0).equals(td.className)) {
                            return true;
                        }
                    }
                }
            }
        }

        return false;
    }

    /**
* Determine whether a class is included in the blacklist.If it is the entire package, then it is determined that this class is contained.If the class has an internal class, if this class contains, then this inner class is also included.
*
* @param list
* The list list.
* @param typeDescription
* @return true: include.false: does not contain.
*/
    public static boolean isMatchClassInBlackList(List<TypeDescription> list, TypeDescription typeDescription) {
        Iterator<TypeDescription> it = list.iterator();
        while (it.hasNext()) {
            TypeDescription td = it.next();

            String packageName = typeDescription.packageName;
            String className = typeDescription.className;

            if (isMatchPackage(td.packageName, packageName)) {
                // If the class name is null, then this item is scoped to the package.
                if (null == td.className) {
                    return true;
                } else {
                    int index_0 = -1;
                    if ("*".equals(td.className)) {
                        if (null == td.methodName)
                            return true;
                    } else if ('*' == td.className.charAt(td.className.length() - 1)) {
                        if (0 == className.indexOf(td.className.substring(0, td.className.length() - 1))) {
                            if (null == td.methodName)
                                return true;
                        }
                    } else if (td.className.equals(className)) {
                        if (null == td.methodName)
                            return true;
                    } else if (-1 != (index_0 = className.indexOf('$'))) {
                        if (className.substring(0, index_0).equals(td.className)) {
                            if (null == td.methodName)
                                return true;
                        }
                    }
                }
            }
        }

        return false;
    }

    /**
* Matching the package name.
* @param packageNameInList
* @param packageName
* @return
*/
    private static boolean isMatchPackage(String packageNameInList, String packageName) {
        if (null == packageNameInList) {
            if ("".equals(packageName))
                return true;
            else
                return false;
        }
        // *Represents a wildcard.
        if ('*' == packageNameInList.charAt(packageNameInList.length() - 1)) {
            if (0 != packageName.indexOf(packageNameInList.substring(0, packageNameInList.length() - 1))) {
                return false;
            } else {
                return true;
            }
        } else if (packageName.equals(packageNameInList)) {
            return true;
        } else if (0 == packageName.indexOf(packageNameInList)) {
            return true;
        }
        return false;
    }

    /**
* Determines whether the method is included in the Black List.If the method name is null, then that contains.If the method name is same but method signature is null, then that contains.
*
* @param list
* List.
* @param typeDescription
* Method reference.
* @return true: include.false: does not contain.
*/
    public static boolean isMatchMethodInWhiteList(List<TypeDescription> list, TypeDescription typeDescription) {
        if (0 == list.size()) {
            return true;
        }
        for (TypeDescription td : list) {
            if (null == td.methodName) {
                return true;
            } else if ('*' == td.methodName.charAt(td.methodName.length() - 1)) {
                if (0 == typeDescription.methodName.indexOf(td.methodName.substring(0, td.methodName.length() - 1))) {
                    return true;
                }
            } else if (td.methodName.equals(typeDescription.methodName)) {
                if (null == td.methodSig) {
                    return true;
                } else if (td.methodSig.equals(typeDescription.methodSig)) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean isMatchMethodInBlackList(List<TypeDescription> list, TypeDescription typeDescription) {
        if (0 == list.size()) {
            return false;
        }
        for (TypeDescription td : list) {
            if (null == td.methodName) {
                return false;
            } else if ('*' == td.methodName.charAt(td.methodName.length() - 1)) {
                if (0 == typeDescription.methodName.indexOf(td.methodName.substring(0, td.methodName.length() - 1))) {
                    return true;
                }
            } else if (td.methodName.equals(typeDescription.methodName)) {
                if (null == td.methodSig) {
                    return true;
                } else if (td.methodSig.equals(typeDescription.methodSig)) {
                    return true;
                }
            }
        }
        return false;
    }

}

