package lz.android.shell.controlcentre;

import com.google.common.io.ByteStreams;
import lz.android.protect.EncodeSection;
import lz.android.protect.encodeFunc;
import lz.android.shell.base.Common;
import lz.android.shell.base.TypeDescription;
import lz.android.shell.base.Utils;
import lz.android.shell.base.ZipHelper;
import lz.android.shell.base.helper.AndroidManifestHelper;
import lz.android.shell.base.helper.CommandHelper;
import lz.android.shell.separator.InstructionInsert01;
import lz.android.shell.separator.Separator;
import lz.android.shell.separator.SeparatorOption;
import org.apache.log4j.Logger;
import org.jf.dexlib2.DexFileFactory;
import org.jf.dexlib2.dexbacked.raw.RawDexFile;

import java.io.*;
import java.nio.file.Files;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;


public class ControlCentre {

    private final static Logger log = Logger.getLogger(ControlCentre.class);

    private ControlCentreOption mOpt;
    private File mApkUnpackDir;

    public ControlCentre(ControlCentreOption opt) throws IOException {
        mOpt = opt;
        prepare();
    }
    /**
* Do some preparation work.
*/
    private void prepare() throws IOException {
       // Files.createTempDirectory()
        // Create a working directory.
        mOpt.workspace = Files.createTempDirectory(mOpt.outDir.toPath(), "advmp").toFile();
        log.info("workspack:" + mOpt.workspace);

        mApkUnpackDir = new File(mOpt.workspace, "apk");
        mApkUnpackDir.mkdir();

        // The entire apk file decompression.
        ZipHelper.unZip(mOpt.apkFile.getAbsolutePath(), mApkUnpackDir.getAbsolutePath());

    }





    /**
* Plus shell.
*
* @return
*/
    public boolean shell(String ProtectClassName,String ProtectMethodName,String InputProtectClassName) {
        boolean bRet = false;
        try {
            // Insert the instruction.
            TypeDescription classDesc = AndroidManifestHelper.findFirstClass(new File(mApkUnpackDir, "AndroidManifest.xml"),InputProtectClassName);
            InstructionInsert01 instructionInsert01 = new InstructionInsert01(new File(mApkUnpackDir, "classes.dex"), classDesc);
            instructionInsert01.insert();
            log.info("insert done!!!");

            // ***********Run away from the controller.
            runSeparator(ProtectClassName,ProtectMethodName);

            // From the template directory copy the jni file.
            copyJniFiles();

            // ********Update jni file content.
            updateJniFiles();

            // Compile the native code.
            buildNative();

            // The libs directory is renamed to lib.
            mOpt.libDir = new File(mOpt.jniDir.getParentFile(), "lib");
            new File(mOpt.jniDir.getParentFile(), "libs").renameTo(mOpt.libDir);

            // Mobile yc file.
            File assetsDir = new File(mApkUnpackDir, "assets");
            if (!assetsDir.exists()) {
                assetsDir.mkdir();
            }
            File newYcFile = new File(assetsDir, "classes.nisl");
            Files.move(mOpt.outYcFile.toPath(), newYcFile.toPath());

            // Mobile classes.dex file.
            System.out.println("mOpt.outYcFile.getParent()"+mOpt.outYcFile.getParent());
            Utils.copyFile(new File(mOpt.outYcFile.getParent(), "classes.dex").getAbsolutePath(), new File(mApkUnpackDir, "classes.dex").getAbsolutePath());
            // Copy the lib directory.
            Utils.copyFolder(mOpt.libDir.getAbsolutePath(), mApkUnpackDir.getAbsolutePath() + File.separator + "lib");

            //TODO encrypt the generated so file in the key function
           // encryption();

            //Delete the METAвЂ”INF folder for re-signing
            File metaFile =new File(mApkUnpackDir.getAbsolutePath()+File.separator+"META-INF");
            Utils.forceDelete(metaFile);

            // Package.
            String name = mOpt.apkFile.getName();
            name = name.substring(0, name.lastIndexOf('.'));
            File outApkFile = new File(mOpt.outDir, name + ".shelled.apk");
            log.info( " outApkFile.getAbsolutePath()"+outApkFile.getAbsolutePath());
            ZipHelper.doZip(mApkUnpackDir.getAbsolutePath(), outApkFile.getAbsolutePath());

            //Delete the temporary file
           // Utils.forceDelete(mOpt.workspace);

            //Re-signed
            reSig();

            bRet = true;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return bRet;
    }

    /**
* Run the extractor.
*
* @return
* @throws IOException
*/
    private boolean runSeparator(String ProtectClassName,String ProtectMethodName) throws IOException {
        SeparatorOption opt = new SeparatorOption();
        opt.dexFile = new File(mApkUnpackDir, "classes.dex");
        File outDir = new File(mOpt.workspace, "separator");
        opt.outDexFile = new File(outDir, "classes.dex");
        opt.outNISLFile = mOpt.outYcFile = new File(outDir, "classes.nisl");
        opt.outCPFile = mOpt.outYcCPFile = new File(outDir, "dexvmp.cpp");
        opt.outHFile=mOpt.outYchFile=new File(outDir,"avmp.h");
        opt.dexBuf=mOpt.dexBuf;
        //opt.whichSDK=Math.random()

        Separator separator = new Separator(opt);
        return separator.run(ProtectClassName,ProtectMethodName);
    }

    /**
* The generated so file in the key function encryption to protect the
*/
    private boolean encryption()
    {
        File parent=new File(mApkUnpackDir.getAbsolutePath() + File.separator + "lib"+File.separator);
        if(parent.exists())
        {
           for(File childFile: parent.listFiles())
           {
               String sopath=childFile.getAbsolutePath()+File.separator+"lib"+Common.SO_NAME+".so";
              // EncodeSection encode=new EncodeSection(sopath,"_Z23NISLvmInterpretPortableP6YcFileiP7_JNIEnvP8_jobjectz");
               encodeFunc encode=new encodeFunc(sopath,"_Z23NISLvmInterpretPortableP6YcFileiP7_JNIEnvP8_jobjectz");
              boolean isEncrypt= encode.encode();
              if(isEncrypt)
                  log.info(sopath +"encrypt success");
           }
        }
      return true;
    }

    /**
* Place the template in the jni directory copied to the working directory.
*
* @throws IOException
*/
    private void copyJniFiles() throws IOException {
        // TODO here written dead.
        File jniTemplateDir = new File(System.getProperty("user.dir") + File.separator + "Tool" + File.separator + "template" + File.separator + "jni");
        mOpt.jniDir = new File(mOpt.workspace, "jni");
        log.info("old jniTemplateDir"+jniTemplateDir.getAbsolutePath());

        Utils.copyFolder(jniTemplateDir.getAbsolutePath(), mOpt.jniDir.getAbsolutePath());
    }

    /**
* Update jni files in the directory.
*/
    private void updateJniFiles() throws IOException {
        File file;
        File tmpFile;
        StringBuffer sb = new StringBuffer();

        // Update avmp.cpp file in the content.
        try (BufferedReader reader = new BufferedReader(new FileReader(mOpt.outYcCPFile))) {
            String line = null;
            while (null != (line = reader.readLine())) {
                sb.append(line);
                sb.append(System.getProperty("line.separator"));//Row delimiter
            }
        }

        file = new File(mOpt.jniDir.getAbsolutePath() + File.separator + "advmpc" + File.separator + "avmp.cpp");
        tmpFile = new File(mOpt.jniDir.getAbsolutePath() + File.separator + "advmpc" + File.separator + "avmp.cpp" + ".tmp");
        try (BufferedReader reader = new BufferedReader(new FileReader(file));
             BufferedWriter writer = new BufferedWriter(new FileWriter(tmpFile))) {
            String line = null;
            while (null != (line = reader.readLine())) {
                if ("#ifdef _AVMP_DEBUG_".equals(line)) {
                    writer.write("#if 0");
                    writer.newLine();
                } else if ("//+${replaceAll}".equals(line)) {
                    writer.write(sb.toString());
                } else {
                    writer.write(line);
                    writer.newLine();
                }
            }
        }
        file.delete();
        tmpFile.renameTo(file);
        sb.delete(0, sb.length());

        //Copy h Files to the jni directory
        Utils.copyFile(mOpt.outYchFile.getAbsolutePath(),mOpt.jniDir.getAbsolutePath()+ File.separator + "advmpc" +File.separator+mOpt.outYchFile.getName());

    }
    /**
* On the final packaged good program Weight signature
* @throws FileNotFoundException
*/
    private void reSig() throws IOException {
        //Copy the packaged application to the apk in the same folder
        File sigjarDir = new File(System.getProperty("user.dir") + File.separator + "Tool" + File.separator + "Signer");
        String name = mOpt.apkFile.getName();
        name = name.substring(0, name.lastIndexOf('.'));
        File unsigapk = new File(mOpt.outDir, name + ".shelled.apk");
        File unsigapkDir =new File(unsigapk.getParent());
        String privateKey="";
        String publicKey="";
        String sigjar="";
        //Cycle the Signer folder all of the files in overwrite to the signature folder
        if (sigjarDir.exists())
        {
            if (sigjarDir.isDirectory())
            {
                for(String list:sigjarDir.list())
                {
                    File tmpfile =new File(unsigapkDir,list);
                    String tmpname=tmpfile.getName();
                   // Utils.copyFile(new File(sigjarDir,list).getAbsolutePath(),tmpfile.getAbsolutePath());
                   String subString =tmpname.substring(tmpname.lastIndexOf('.'),tmpname.length());
                    if (tmpname.substring(tmpname.lastIndexOf('.')+1,tmpname.length()).equals("pk8"))
                        privateKey=tmpname;
                    if (tmpname.substring(tmpname.lastIndexOf('.')+1,tmpname.length()).equals("pem"))
                        publicKey=tmpname;
                    if (tmpname.substring(tmpname.lastIndexOf('.')+1,tmpname.length()).equals("jar"))
                        sigjar=tmpname;

                }
            }else
                return;
        }else
            return;

        //The call signature of a jar be re-signed

       File privateKeyFile = new File(sigjarDir,privateKey);
        File publicKeyFile =new File(sigjarDir,publicKey);
        File sigjarFile =new File(sigjarDir,sigjar);
        if (privateKeyFile.exists()&&publicKeyFile.exists()&&sigjarFile.exists()) {
              File sigApk =new File(unsigapkDir,name+"_siged.apk");
            log.info("------ Start signature------");
            CommandHelper.exec(new String[]{"java","-jar",sigjarFile.getAbsolutePath(),publicKeyFile.getAbsolutePath(),privateKeyFile.getAbsolutePath(),unsigapk.getAbsolutePath(),sigApk.getAbsolutePath()});
            log.info("------ Signature end------");
            //Rename
            if(unsigapk.exists())
            {
                unsigapk.delete();
                sigApk.renameTo(unsigapk);
            }
        } else {
            throw new FileNotFoundException("The signature file is not complete, not signed");
        }


    }


    /**
* Compile the native code.
* @throws FileNotFoundException
*/
    private void buildNative() throws FileNotFoundException {
        String ndkpathname ="NDK_HOME";
        String ndkpath="";
        Map<String,String> m = System.getenv();
        for(String key: m.keySet()){
            System.out.println(key+"  "+m.get(key));
            if ( ndkpathname.equals(key)) {
                ndkpath=m.get(key);
                break;
            }
        }
        File ndkDir= new File("D:\\env\\android-ndk-r10d");
        //log.info("ndkDir"+ndkDir);
        if (ndkDir.exists()) {
            String ndkPath = ndkDir.getAbsolutePath() + File.separator + "ndk-build";
            if (Utils.isWindowsOS()) {
                ndkPath += ".cmd";
            }
            log.info("------ Start to compile native code------");
            // Compile the native code.
            CommandHelper.exec(new String[]{ndkPath, "NDK_PROJECT_PATH=" + mOpt.jniDir.getParent()});
            log.info("------ The end of the compilation------");
        } else {
            throw new FileNotFoundException("The failure to pass the environment variable\"ndk\"Find ndk directory! This environment variable may not be set.");
        }
    }

}

