package lz.android.shell.separator;

import lz.android.dexlib2.helper.MethodHelper;
import lz.android.shell.advmpformat.StringItem;
import lz.android.shell.advmpformat.NISLFile;
import lz.android.shell.advmpformat.Format;
import lz.android.shell.base.Common;
import lz.android.shell.separator.config.Config;
import lz.android.shell.separator.config.ConfigHelper;
import lz.android.shell.separator.config.ConfigParse;
import org.jf.dexlib2.AccessFlags;
import org.jf.dexlib2.DexFileFactory;
import org.jf.dexlib2.dexbacked.DexBackedMethod;
import org.jf.dexlib2.iface.ClassDef;
import org.jf.dexlib2.iface.DexFile;
import org.jf.dexlib2.iface.Method;
import org.jf.dexlib2.iface.MethodParameter;
import org.jf.dexlib2.immutable.ImmutableMethod;
import org.jf.dexlib2.immutable.ImmutableMethodParameter;
import org.jf.dexlib2.rewriter.*;
import lz.android.shell.processins.*;
import javax.annotation.Nonnull;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
* Method of instruction extractor.
*/
public class Separator {
    public static DexFile mDexFile;
    private ConfigHelper mConfigHelper;
    private DexRewriter mDexRewriter;
    private SeparatorOption mOpt;

    private List<Method> mSeparatedMethod = new ArrayList<>();//mSepatedMethod used to record the entire of the extracted function
    private List<Format.SeparatorData> mSeparatorData = new ArrayList<>();
    private String ProtectingClassName = null;
    private String ProtectingMethodName = null;

    /**
* @param opt
*/
public Separator(SeparatorOption opt) throws IOException {
mOpt = opt;
//mOpt.whichSDK=(int) (Math.random()*4);//random from the four sets of instructions to select a set of
mOpt.whichSDK=3;

mDexFile = DexFileFactory.loadDexFile(opt.dexFile, Common.API); // This dex file is already modified, the
SeparatorRewriterModule srm = new SeparatorRewriterModule();
mDexRewriter = new SeparatorDexRewriter(srm);



ConfigParse cfg = new ConfigParse(opt.configFile);
Config config = cfg.parse();
//opt.configFile is empty!!!

// Parsing the configuration file.
mConfigHelper = new ConfigHelper(config);

if (! mOpt.outDexFile.exists()) {
mOpt.outDexFile.getParentFile().mkdirs();
}
}

/**
* Extraction method instruction.
*
* @return true: success.false: fail.
*/
    public boolean run(String ProtectClassName,String ProtectMethodName) {

        boolean bRet = false;

        ProtectingClassName = ProtectClassName;
        ProtectingMethodName = ProtectMethodName;
        // Rewriting the dex file
         DexFile newDexFile = mDexRewriter.rewriteDexFile(mDexFile);
        try {
            // The new dex output to a file.
            DexFileFactory.writeDexFile(mOpt.outDexFile.getAbsolutePath(), newDexFile);

            // Write NISL file.
            writeNISLFile();

            // Write c and h Files.
            writeCFileandH();

            bRet = true;
        } catch (IOException e) {
            e.printStackTrace();
        }
        return bRet;
    }

    /**
* Write NISL file.
*
* @throws IOException
*/
    private void writeNISLFile() throws IOException {
        Format format = new Format();
        // TODO no methods field assignment.
        format.methods =new ArrayList<>();
        for(int i=0;i<mSeparatedMethod.size();i++)
        {
            Format.AdvmpMethod method =new Format.AdvmpMethod();
            method.accessFlag =mSeparatedMethod.get(i).getAccessFlags();
            method.definingClass = new StringItem();
            method.definingClass.str =mSeparatedMethod.get(i).getDefiningClass().getBytes();
            method.definingClass.size=method.definingClass.str.length;
            // method.methodIndex=0;
            method.name=new StringItem();
            method.name.str=mSeparatedMethod.get(i).getName().getBytes();
            method.name.size=method.name.str.length;
            method.sig=new StringItem();
            // method.sig=mSeparatedMethod.get(i).getParameters()
            //TODO not write Method
            //format.methods.add(method);
        }
       // ToAdvmpMehod(format.methods);
        format.separatorDatas = mSeparatorData;
        format.stringDatas=ProcessIns.mstringDatas;
        format.typeItemDates=ProcessIns.mTypeItemDates;
        format.methodItemDates=ProcessIns.mMethodItemDates;
        format.fieldItemDates=ProcessIns.mfieldItemDates;
        NISLFile NISLFile = new NISLFile(mOpt.outNISLFile, format);
        NISLFile.write();
    }

    /**
* Write a C file.
*/
    private void writeCFileandH() throws IOException {
        //TODO
        int whichSDK=mOpt.whichSDK;
        SeparatorHWriter separatorHWriter=new SeparatorHWriter(whichSDK,mOpt.outHFile);
        separatorHWriter.write();
        SeparatorCWriter separatorCWriter = new SeparatorCWriter(mOpt.outCPFile, mSeparatedMethod,whichSDK);
        separatorCWriter.write();
        System.out.println(mOpt.outCPFile.getAbsolutePath());
    }

    class SeparatorDexRewriter extends DexRewriter {

        public SeparatorDexRewriter(RewriterModule module) {
            super(module);//Call the rewritten
        }

        @Nonnull
        @Override
        public DexFile rewriteDexFile(@Nonnull DexFile dexFile) {
            return super.rewriteDexFile(dexFile);
        }
    }

    class SeparatorRewriterModule extends RewriterModule {
        @Nonnull
        @Override
        public Rewriter<ClassDef> getClassDefRewriter(@Nonnull Rewriters rewriters) {
            return new ClassDefRewriter(rewriters) {
                @Nonnull
                @Override
                public ClassDef rewrite(@Nonnull ClassDef classDef) {
                    return super.rewrite(classDef);
                }
            };
        }

      //TODO temporarily modify the
        int flag=2;
        @Nonnull
        @Override
        public Rewriter<Method> getMethodRewriter(Rewriters rewriters) {
            //
            return new MethodRewriter(rewriters) {
                @Nonnull
                @Override
                public Method rewrite(@Nonnull Method value) {
                    System.out.println("methodName "+value.getName()+" Parameters "+value.getParameters()+" tDefiningClass "+value.getDefiningClass()+" ReturnType "+value.getReturnType());
//                   String addMethod ="separatorTest";
//                    if(value.getName().equals(addMethod))
//                    System.out.println("separatorTest method find");

                    if (mConfigHelper.isValid(value,ProtectingClassName,ProtectingMethodName)) {
                        mSeparatedMethod.add(value);//Records are separated out of the Method
                        System.out.println("separatorTest the code extract in");
                        Format.SeparatorData separatorData = new Format.SeparatorData();
                        separatorData.methodIndex = mSeparatorData.size();//Identification is the first of several of the extracted function


                        separatorData.accessFlag = value.getAccessFlags();
                        separatorData.paramSize = value.getParameters().size();
                        //Get the size of the registers
                        separatorData.registerSize = value.getImplementation().getRegisterCount();

                        //The method of the short type is described using the section prefix description
                        separatorData.paramShortDesc = new StringItem();
                        separatorData.paramShortDesc.str = MethodHelper.genParamsShortDesc(value).getBytes();
                        separatorData.paramShortDesc.size = separatorData.paramShortDesc.str.length;
                        System.out.println("separatorData.paramShortDesc.str   " + separatorData.paramShortDesc.str.toString());
                        System.out.println("separatorData.paramShortDesc.size   " + separatorData.paramShortDesc.size);
                        separatorData.insts = MethodHelper.getInstructions((DexBackedMethod) value);
                        System.out.println("The method of instruction as follows:");
                        for (short ins : separatorData.insts) {
                            int shortValue = ins & 0x7fff + (Math.abs(ins >> 15) << 15);
                            String shortValue1 = Integer.toHexString(shortValue);
                            System.out.print(shortValue1 + " ");
                        }
                        System.out.println();
                        ProcessIns pi = new ProcessIns(separatorData, mOpt);
                        separatorData.insts = pi.processing();//Re-insts assignment
                        separatorData.instSize = separatorData.insts.length;
                        separatorData.size = 4 + 4 + 4 + 4 + 4 + separatorData.paramShortDesc.size + 4 + (separatorData.instSize * 2) + 4;


                        mSeparatorData.add(separatorData);//Each reincarnation a plus one

                        // Following doing the purpose is to put the method name is removed, otherwise the generation of the dex install when there will be this error: INSTALL_FAILED_DEXOPT it.
                        List<? extends MethodParameter> oldParams = value.getParameters();
                        List<ImmutableMethodParameter> newParams = new ArrayList<>();
                        for (MethodParameter mp : oldParams) {
                            newParams.add(new ImmutableMethodParameter(mp.getType(), mp.getAnnotations(), null));
                        }
                        //new ImmutableMethod(value.getDefiningClass(), value.getName(), newParams, value.getReturnType(), value.getAccessFlags() | AccessFlags.NATIVE.getValue(), value.getAnnotations(), null);

                        //If Yes return a new method, if not return the parent class rewrite
                        return new ImmutableMethod(value.getDefiningClass(), value.getName(), newParams, value.getReturnType(), value.getAccessFlags() | AccessFlags.NATIVE.getValue(), value.getAnnotations(), null);
                    }

                    return super.rewrite(value);
                }
            };
        }
    }

}

